/*
 * The Program materials contained in this file are IBM copyright materials.
 * COPYRIGHT International Business Machines Corp. 2015
 * All Rights Reserved * Licensed Materials - Property of IBM
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

var core = (function() {
	
	// Flag indicates if running in a collective environment
	// Possible values: constants constants.ENVIRONMENT_COLLECTIVE, constants.ENVIRONMENT_LOCAL, or null (not determined)
	var environment = null;
	
	// Id of the server that hosts the tool
	var hostServerId = null;
	
	// Server id (when working in a collective environment)
	var serverId = null;
	
	// List of servers that have been checked
	var checkedServerIds = [];
	
	// Flag to keep track of server data initialization
	var serverDataInitialized = false;
	
	// Server schema in text format
	var serverSchemaInTextFormat = null;
	
	// Flag indicated if config tool is embedded
	// Possible values: constants constants.EMBEDDED_ADMIN_CENTER, constants.EMBEDDED_EXPLORE_TOOL, or null (not embedded)
	var embedded = window.frameElement != null? (window.frameElement.id == "exploreContainerForConfigTool"? constants.EMBEDDED_EXPLORE_TOOL : constants.EMBEDDED_ADMIN_CENTER) : null;

	// Flag for blocking UI
	var uiBlocked = false;

	var startup = function() {
		
		// Update title
		setNavbarTitleText(editorMessages.LOADING);
		
		// Hide all controls
		hideAllControls();
		
		// Show progress
		showControlById("progress", true);
		
		// Obtain information from URL
		var dataFromURL = getDataFromURLHash();
		
		// Check if running in a collective environment
		checkCollectiveEnvironment().done(function() {
			
			// Obtain host server id
			retrieveHostServerId().done(function(retrievedHostServerId) {
				
				hostServerId = retrievedHostServerId;
				
				if(dataFromURL.serverId != null) {
					
					// Check if the server id provided is valid
					checkServerId(dataFromURL.serverId).done(function() {
						
						var forceServerRefresh = false;
						
						if(serverId != null && serverId != dataFromURL.serverId) {
							forceServerRefresh = true;
						}
						
						// Update serverId
						serverId = dataFromURL.serverId;
						
						// Open or list files
						openOrListFiles(dataFromURL.filePath, forceServerRefresh);
						
					}).fail(function() {
						
						// Show error message
						renderServerNotFoundErrorMessage(dataFromURL.serverId);
						
						// Hide progress
						hideControlById("progress");
						
						// Update title
						setNavbarTitleText(editorMessages.ERROR);
						
						// Show change server button
						if(embedded != constants.EMBEDDED_EXPLORE_TOOL) {
							showControlById("navbarChangeServerSection");
						}
						
					});
				} else if(dataFromURL.filePath != null) {
					
					// Local reference in collective environment
					openOrListFiles(dataFromURL.filePath, false);
					
				} else {
					
					// Render list of servers
					renderServerList();	
				}
				
			}).fail(function() {
				
				// Show error message
				core.renderMessage(editorMessages.COULD_NOT_RETRIEVE_SERVER_IDENTIFICATION, "danger");
				
				// Hide progress
				hideControlById("progress");
				
				// Update title
				setNavbarTitleText(editorMessages.ERROR);
				
			});
			
		}).fail(function() {
			
			if(dataFromURL.serverId != null) {
				
				// Update title
				setNavbarTitleText("");
				
				// Hide progress
				core.hideControlById("progress", false, false);
				
				// Show error message (server referenced but not in a collective environment)
				renderCannotAccessRemoteServerFromNoneCollectiveControllerMessage(dataFromURL.serverId);
				
			} else {
				
				// Open or list files
				openOrListFiles(dataFromURL.filePath, false);
				
			}
		});
	};
	
	
	var checkCollectiveEnvironment = function() {
		var deferred = new $.Deferred();
		if(environment == null) {
			collectiveUtils.checkCollectiveEnvironment().done(function() {
				
				// Update collective flag
				environment = constants.ENVIRONMENT_COLLECTIVE;
				
				deferred.resolve();
			}).fail(function() {
				
				// Update collective flag
				environment = constants.ENVIRONMENT_LOCAL;
				
				deferred.reject();
			});
		} else if(environment == constants.ENVIRONMENT_COLLECTIVE) {
			deferred.resolve();
		} else if(environment == constants.ENVIRONMENT_LOCAL) {
			deferred.reject();
		}
		return deferred;
	};	
	
	
	var retrieveHostServerId = function() {
		if(hostServerId != null) {
			return new $.Deferred().resolve(hostServerId);
		} else {
			return collectiveUtils.retrieveHostServerId();
		}
	};
	
	
	var checkServerId = function(serverId) {
		var deferred = new $.Deferred();
		if(checkedServerIds.indexOf(serverId) != -1) {
			deferred.resolve();
		} else {
			collectiveUtils.checkServerId(serverId).done(function() {
				checkedServerIds.push(serverId);
				deferred.resolve();
			}).fail(function() {
				deferred.reject();
			});
		}
		return deferred;
	};

	
	var openOrListFiles = function(filePath, forceServerRefresh) {
		
		// Initialize server information
		retrieveServerData(forceServerRefresh).done(function() {
			
			if(filePath != null) {
				// Attempt to retrieve file
				fileUtils.retrieveFile(filePath).done(function(file) {
					
					// Open file
					openFileForEditing(file);
					
				}).fail(function() {
					
					// Show error message
					var errorMessage = stringUtils.formatString(editorMessages.FILE_NOT_FOUND, [filePath]);
					renderMessage(errorMessage, "danger", true);

					// List files
					renderFileList();
					
				});
			} else {
				
				// List files
				renderFileList();
			}
			
		}).fail(function() {
			
			// Update title
			setNavbarTitleText("");
			
			// Hide progress
			core.hideControlById("progress", false, false);
			
			// Show error message
			renderMessage(editorMessages.ERROR_RETRIEVING_SERVER_INFORMATION, "danger", false);

			// Show change server option if running in a collective environment and not embedded in the explore tool
			if(environment == constants.ENVIRONMENT_COLLECTIVE && embedded != constants.EMBEDDED_EXPLORE_TOOL) {
				core.showControlById("navbarChangeServerSection", false);
			}
		});
	};

	
	var renderFileList = function() {
		
		fileExplorer.renderFileExplorer().done(function() {

			// Show file explorer
			core.showControlById("fileExplorer", false);
			
		}).fail(function() {
			
			// Show error message
			renderMessage(editorMessages.FILE_ACCESS_ERROR_MESSAGE, "danger", false);
			
		}).always(function() {
			
			// Update title
			if(environment == constants.ENVIRONMENT_COLLECTIVE && embedded != constants.EMBEDDED_EXPLORE_TOOL) {
				setNavbarTitleText(serverId.substring(serverId.lastIndexOf(",") + 1));
			} else {
				setNavbarTitleText(editorMessages.CONFIGURATION_FILES);	
			}
			
			// Hide progress
			core.hideControlById("progress", false, false);
			
			// Show change server button on navigation bar
			if(environment == constants.ENVIRONMENT_COLLECTIVE && embedded != constants.EMBEDDED_EXPLORE_TOOL) {
					
				core.showControlById("navbarChangeServerSection", false);
				
			} else {
				
				// Show sign out button on navigation bar
				if(!isInIFrame()) {
					core.showControlById("navbarSignOutSection", false);
				}
			}
			
			// Focus title for accessibility compliance
			$("#navigationBarTitle").focus();
			
		});
	};
	
	
	var renderServerNotFoundErrorMessage = function(serverId) {
		// show error message of server not found
		var firstCommaIndex = serverId.indexOf(",");
		var lastCommaIndex = serverId.lastIndexOf(",");

		var serverName = serverId.substring(lastCommaIndex + 1, serverId.length);
		var hostName = serverId.substring(0, firstCommaIndex);
		var userDir = serverId.substring(firstCommaIndex + 1, lastCommaIndex);

		var errorMessage = stringUtils.formatString(editorMessages.SERVER_NOT_FOUND, [serverName, hostName, userDir]);
		core.renderMessage(errorMessage, "danger");	
	};

	
	var renderCannotAccessRemoteServerFromNoneCollectiveControllerMessage = function() {
		var link = "<a href=\"\" draggable=\"false\" onclick=\"core.run('');\">" + editorMessages.HERE + "</a>";
		var errorMessage = stringUtils.formatString(editorMessages.ERROR_NOT_IN_COLLECTIVE_ENVIRONMENT, [link]);	
		renderMessage(errorMessage, "danger", true);
	};


	var renderServerList = function() {
		serverExplorer.renderServerExplorer().done(function() {
			
			// Update title
			setNavbarTitleText(editorMessages.SELECT_SERVER);
			
			// Show sign out button on navigation bar
			if(!isInIFrame()) {
				showControlById("navbarSignOutSection", false);
			}
			
			// Show server explorer
			showControlById("serverExplorer", false);
			
		}).fail(function() {
			
			// Show error message
			renderMessage(editorMessages.ERROR_ACCESSING_SERVER_LIST, "danger", true);
		}).always(function() {
			
			// Hide progress
			core.hideControlById("progress", false, false);
			
			// Focus title for accessibility compliance
			$("#navigationBarTitle").focus();
			
		});
	};
	
	
	var openFileForEditing = function(file) {
		
		// Obtain file name for title
		var index = file.path.lastIndexOf("/");
		if(index == -1) {
			index = file.path.lastIndexOf("\\");
		}
		var fileName = file.path.substring(index + 1);
		
		// Update title
		setNavbarTitleText(fileName);
		
		// Show editor buttons on navigation bar
		showControlById("navbarEditorSection", false);
		
		editor.openFileForEditing(file, serverSchemaInTextFormat);
		
		showControlById("editor", false);
		
		// Hide progress
		core.hideControlById("progress", false, false);
		
		// Focus title for accessibility compliance
		$("#navigationBarTitle").focus();
	};
	
	
	var getDataFromURLHash = function() {
		
		var serverId = null;
		var filePath = null;

		var hash = null;
		if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
			hash = window.location.hash;
		} else {
			hash = window.top.location.hash;
		}
		
		if(hash != null && hash.length > 14 && hash.substring(0, constants.SERVER_CONFIG_HASH.length + 2) == "#" + constants.SERVER_CONFIG_HASH + "/") {
				
			var value = hash.substring(14);
			var lastcommaIndex = value.lastIndexOf(",");
			
			if(lastcommaIndex > 0) {
				var slashIndex = value.indexOf("/", lastcommaIndex + 1);
				if(slashIndex != -1) {
					serverId = value.substring(0, slashIndex);
					filePath = value.length > slashIndex + 1? value.substring(slashIndex + 1) : null;
				} else {
					serverId = value;
				}
			} else {
				filePath = value;
			}
		}
		return {
			serverId: serverId,
			filePath: filePath
		};

	};

	
	var setNavbarTitleText = function(title) {
		$("#navigationBarTitle").text(title);
	};

	
	/**
	 * messageType: "success" or "info" (default) or "warning" or "danger" 
	 */
	var renderMessage = function(messageText, messageType, dismissable, clearPreviousMessages) {
		var messageContainer = $("#messageContainer");
		if(clearPreviousMessages) {
			messageContainer.empty();
		}
		var alert = $("<div class=\"alert\" role=\"alert\" />");	
		if(!messageType) {
			messageType = "info";
		}
		alert.addClass("alert-" + messageType);
		if(dismissable) {
			alert.addClass("alert-dismissable");
			alert.append("<button type=\"button\" class=\"close\" data-dismiss=\"alert\" aria-label=\"Close\"><span aria-hidden=\"true\">&times;</span></button>");
		}
		alert.append(messageText);
		messageContainer.prepend(alert);
	};
	
	
	var showControlById = function(id, animate, timeUnits) {
		if(!timeUnits) {
			timeUnits = 2;
		}
		var deferred = new $.Deferred();
		var control = $("#" +  id);
		control.stop();
		if(animate) {
			control.css("opacity", "0");
			control.removeClass("hidden");
			control.animate({
				opacity: 1
			}, timeUnits * constants.ANIMATION_TIME_UNIT, function() {
				deferred.resolve();
			});
		} else {
			control.stop();
			control.css("opacity", "1");
			control.removeClass("hidden");
			deferred.resolve();
		}
		return deferred;
	};
	
	
	var hideControlById = function(id, animate, preserveSpace) {
		var deferred = new $.Deferred();
		var control = $("#" +  id);
		control.stop();
		if(animate) {
			control.animate({
				opacity: 0
			}, 3 * constants.ANIMATION_TIME_UNIT, function() {
				if(!preserveSpace) {
					control.addClass("hidden");
				}
				deferred.resolve();
			});
		} else {
			if(preserveSpace) {
				control.css("opacity", "0");
			} else {
				control.addClass("hidden");
			}
			deferred.resolve();
		}
		return deferred;
	};
	
	
	var hideAllControls = function() {
		$("#messageContainer").empty();
		hideControlById("navbarChangeServerSection", false, false);
		hideControlById("navbarEditorSection", false, false);
		hideControlById("navbarSignOutSection", false, false);
		hideControlById("serverExplorer", false, false);
		hideControlById("fileExplorer", false, false);
		hideControlById("editor", false, false);
		hideControlById("navbarEditorChangesSavedMessage", false, false);
	};
	
	
	var retrieveServerData = function(forceServerRefresh) {
		var deferred = new $.Deferred();
		if(!serverDataInitialized || forceServerRefresh) {
			var retrieveServerWritePathsPromise = fileUtils.retrieveServerWritePaths();
			var retrieveServerVariableValuesPromise = fileUtils.retrieveServerVariableValues();
			var retrieveSchemaFilePromise = fileUtils.retrieveSchemaFile();
			$.when(retrieveServerWritePathsPromise, retrieveServerVariableValuesPromise, retrieveSchemaFilePromise).done(function(serverWritePaths, serverVariableValues, schemaFile) {
				serverDataInitialized = true;
				serverSchemaInTextFormat = schemaFile;
				deferred.resolve();
			}).fail(function() {
				deferred.reject();
			});
		} else {
			deferred.resolve();
		}
		return deferred;
	};
	
	var updateNavigation = function(hash) {
		if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
//			window.history.pushState(null, null, hash);
		} else {
			window.top.history.pushState(null, null, hash);
		}
	};
	
	var getFileURL = function(filePath) {
		var baseURL = window.top.location.href.split("#")[0];
		if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
			if(environment == constants.ENVIRONMENT_COLLECTIVE && serverId != null) {
				return baseURL + "#" + constants.EXPLORE_TOOL_HASH  + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + "/" + filePath;
			} else {
				return baseURL + "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + "/" + filePath;
			}
		} else {
			var hash = "#" + constants.SERVER_CONFIG_HASH + "/";
			if(environment == constants.ENVIRONMENT_COLLECTIVE && serverId != null) {
				hash = hash + serverId + "/";
			}
			return baseURL + hash + filePath;
		}
	};

	
	var setFile = function(filePath) {
		if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
			if(environment == constants.ENVIRONMENT_COLLECTIVE) {
//				window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH  + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT  + "/" + filePath);
				window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH  + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT  + "/" + filePath;
			} else {
//				window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + "/" + filePath);
				window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + "/" + filePath;
			}
		}
		
		var hash = "#" + constants.SERVER_CONFIG_HASH + "/";
		if(environment == constants.ENVIRONMENT_COLLECTIVE && serverId != null) {
			hash = hash + serverId + "/";
		}
		hash = hash + filePath;
		updateNavigation(hash);
		startup();
	};
	
	
	var clearFile = function() {
		if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
			if(environment == constants.ENVIRONMENT_COLLECTIVE) {
//				window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT );
				window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT ;
			} else {
//				window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT);
				window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT;
			}
		}
		
		var hash = "#" + constants.SERVER_CONFIG_HASH + "/";
		if(environment == constants.ENVIRONMENT_COLLECTIVE && serverId != null) {
			hash = hash + serverId;
		}
		updateNavigation(hash);
		startup();
	};
	
	
	var setServer = function(serverId) {
		updateNavigation("#" + constants.SERVER_CONFIG_HASH + "/" + serverId);
		startup();
	};
	
	
	var clearServer = function() {
		updateNavigation("#" + constants.SERVER_CONFIG_HASH + "/");
		startup();
	};

	
	var applyRoutingHeaders = function(xhr) {
		if(core.collectiveRoutingRequired()) {
			
			var serverId = core.getCollectiveServerId();
			
			var firstCommaIndex = serverId.indexOf(",");
			var lastCommaIndex = serverId.lastIndexOf(",");
	
			var hostName = serverId.substring(0, firstCommaIndex);
			var userDir = serverId.substring(firstCommaIndex + 1, lastCommaIndex);
			var serverName = serverId.substring(lastCommaIndex + 1, serverId.length);
			
			xhr.setRequestHeader("com.ibm.websphere.jmx.connector.rest.routing.hostName", hostName);
			xhr.setRequestHeader("com.ibm.websphere.jmx.connector.rest.routing.serverName", serverName);
			xhr.setRequestHeader("com.ibm.websphere.jmx.connector.rest.routing.serverUserDir", userDir);
		}		
	};
	
	
	var centerDialog = function(modalElement) {
		var modal = $(modalElement);
		dialog = modal.find(".modal-dialog");
		modal.css("display", "block");
		dialog.css("margin-top", 40 + Math.max(0, ($(window).height() - dialog.height()) / 4));
	};
	
	
	var setUIBlocked = function(blocked) {
		uiBlocked = blocked;
		if(uiBlocked) {
			showControlById("uiBlock");
		} else {
			hideControlById("uiBlock");
		}
	};
	
	
	var isInIFrame = function() {
		return window.frameElement != null;
	};
	
	
	var encodeServerId = function(serverId) {
		var firstCommaIndex = serverId.indexOf(",");
		var lastCommaIndex = serverId.lastIndexOf(",");

		var hostName = serverId.substring(0, firstCommaIndex);
		var userDir = serverId.substring(firstCommaIndex + 1, lastCommaIndex);
		var serverName = serverId.substring(lastCommaIndex + 1, serverId.length);
		
		return hostName + "," + encodeURIComponent(userDir) + "," + serverName;
	};

	$(document).ready(function() {
		
		$(window).on("popstate", function() {
			var skip = false;
			if(embedded != null && window.top.location.hash.length == 0) {
				skip = true;
			}
			if(!skip) {
				if(embedded == constants.EMBEDDED_EXPLORE_TOOL) {
					var filePath = getDataFromURLHash().filePath;
					if(environment == constants.ENVIRONMENT_COLLECTIVE) {
//						window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + (filePath != null? "/" + filePath : ""));
						window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH + "/" + constants.EXPLORE_TOOL_SERVERS_SEGMENT + "/" + serverId + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + (filePath != null? "/" + filePath : "");
					} else {
//						window.top.history.pushState(null, null, "#" + constants.EXPLORE_TOOL_HASH  + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + (filePath != null? "/" + filePath : ""));
						window.top.location.hash = "#" + constants.EXPLORE_TOOL_HASH  + "/" + constants.EXPLORE_TOOL_CONFIGURE_SEGMENT + (filePath != null? "/" + filePath : "");
					}
				} 
				startup();
			}
		});	
		
		$(window).on("keydown", function(event) {
			if(uiBlocked && event.keyCode != 116) {
				return false;
			}
		});
		
	});
	

	return {
		messages: null,
		startup: startup,
		renderMessage: renderMessage,
		showControlById: showControlById,
		hideControlById: hideControlById,
		setNavbarTitleText: setNavbarTitleText,
		centerDialog: centerDialog,
		setServer: setServer,
		clearServer: clearServer,		
		setFile: setFile,
		clearFile: clearFile,
		setUIBlocked: setUIBlocked,
		getFileURL: getFileURL,
		collectiveRoutingRequired: function() {
			return environment == constants.ENVIRONMENT_COLLECTIVE && serverId != hostServerId;
		},
		getCollectiveServerId: function() {
			return serverId;
		},
		encodeServerId: encodeServerId,
		applyRoutingHeaders: applyRoutingHeaders
	};
	
})();
